/**********************************************************************
 *
 * Filename:    sissdk_ecdsa.c
 *
 * Description: Fast use of AES128 CBC
 *
 * Notes:       The parameters for each supported AES standard are
 *				defined in the header file aes.h.  The implementations
 *				here should stand up to further additions to that list.
 *
 *
 * Copyright (c) 2019 by ICWORKSHOP.  This software is placed into
 * the public domain and may be used for any purpose.  However, this
 * notice must not be changed or removed and no warranty is either
 * expressed or implied by its publication or distribution.
 **********************************************************************/

/**
 * @file    aes_wrap.c
 * @brief   Implementation of aes_wrap.h
 */
 
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "sissdk_ecdsa.h"
#include "sissdk_utils.h"

uint8_t  m_grp[CERTIFICATION_ID_SIZE]	= {0x03,00,0x13};

static uint8_t	m_hash[SHA256_HASH_SIZE];

/**
 * \brief  Generate a Hash for the data
 *
 * \param src      Data pointer to the hash is required
 * \param size     Data size
 *                 
 *
 * \return         true on success.
 * \return         false on failure.
 */
 
uint8_t * sissdk_ecdsa_sha256(const uint8_t * src,uint32_t size)
{
	int ret = -1;
	ret = ret;	//to disable warning
	if(src == NULL) {
		sis_log(LogError,"src data is NULL\r\n");
		return NULL;
	}
	if ((ret = mbedtls_sha256_ret(src, size, m_hash, 0)) != 0)
	{
			sis_log(LogError,"[failed]: mbedtls_sha256_ret returned %d\r\n", ret);
			return NULL;
	}
	util_dump_buf("[ok]: Hash = ", m_hash, sizeof(m_hash));
	return m_hash;
}

/**
 * \brief  Load the group
 *
 * \param ctx      The context of the ecdsa
 *                 
 * \return         true on success.
 * \return         false on failure.
 */
bool sissdk_ecdsa_load_group(mbedtls_ecdsa_context * ctx)
{
		int ret = 0;
		ret = ret;	//to disable warning
	
		const uint8_t * group = m_grp;
	    
		if ( 0 != (ret = mbedtls_ecp_tls_read_group(&ctx->grp, &group, CERTIFICATION_ID_SIZE)))
		{
				sis_log(LogError,"[failed]: mbedtls_ecp_tls_read_group: %d\r\n", ret);
				return false;
		}
		return true;
}
/**
 * \brief  Load the public key
 *
 * \param keyPair  The public or private key generated by ecdsa.exe
 * \param ctx      The context of the ecdsa
 *                 
 *
 * \return         true on success.
 * \return         false on failure.
 */

bool sissdk_ecdsa_load_publickey(const uint8_t * pubkey,mbedtls_ecdsa_context * ctx)
{
		int ret = 0;
		ret = ret;	//to disable warning
		
		if(false == sissdk_ecdsa_load_group(ctx))	return false;	//load group
		
    if (0 != (ret = mbedtls_ecp_point_read_binary(&ctx->grp, &ctx->Q, pubkey, CERTIFICATION_PUB_SIZE)))
		{
       sis_log(LogError,"[failed]: mbedtls_ecp_point_read_binary: %d\r\n", ret);
       return false;
    }
    return true;
}

bool sissdk_ecdsa_load_privatekey(const uint8_t * prikey,mbedtls_ecdsa_context * ctx)
{
		int ret = 0;
		ret = ret;	//to disable warning
		
		if(false == sissdk_ecdsa_load_group(ctx))	return false;	//load group
		
		if (0 != (ret = mbedtls_mpi_read_binary(&ctx->d, prikey, CERTIFICATION_PRI_SIZE)))
		{
				sis_log(LogError,"[failed]: mbedtls_mpi_read_binary: %d\r\n", ret);
				return false;
		}
    
    return true;
}


/**
 * \brief  Certificate to verify data
 *
 * \param pubKey  	The public key generated by ecdsa.exe
 * \param src      	The data used to validate the certificate
 * \param src_size  data size
 * \param sig			  Certificate of data
 *                 
 *
 * \return         true on success.
 * \return         false on failure.
 */

bool sissdk_ecdsa_verify(const uint8_t * pubKey,const uint8_t * src,uint32_t src_size,const	S_ECDSA_SIGN * sig)
{
		uint8_t * pHash = NULL;
	
    if(NULL  == (pHash = sissdk_ecdsa_sha256(src,src_size))){
			return false;
		}
		
		mbedtls_ecdsa_context ctx_verify;
		
		mbedtls_ecdsa_init(&ctx_verify);
		if(!sissdk_ecdsa_load_publickey(pubKey,&ctx_verify)){
			mbedtls_ecdsa_free(&ctx_verify);
			return false;
		}
		
		sis_log(LogDebug,"[-v]Preparing verification context...\r\n");
		
		int ret = 0;
		ret = ret;	//to disable warning
    if ((ret = mbedtls_ecdsa_read_signature(&ctx_verify,
        pHash, SHA256_HASH_SIZE,
        sig->m_sign_data, sig->m_sign_size)) != 0)
    {
        sis_log(LogError,"[failed]:mbedtls_ecdsa_genkey returned %d\r\n", ret);
				mbedtls_ecdsa_free(&ctx_verify);
				return false;
    }
    sis_log(LogDebug,"[ok]: verification passed...\r\n");
    mbedtls_ecdsa_free(&ctx_verify);
		
		return true;
}

/**
 * \brief  Generate certificates for the data
 *
 * \param priKey  	The private key generated by ecdsa.exe
 * \param src      	The data used to validate the certificate
 * \param src_size  data size
 * \param sig			  Certificate of data (output)
 *                 
 *
 * \return         true on success.
 * \return         false on failure.
 */
bool sissdk_ecdsa_sign(const uint8_t * priKey,const uint8_t * src,uint32_t src_size,S_ECDSA_SIGN * sig)
{
		uint8_t * pHash = NULL;
    if(NULL  == (pHash = sissdk_ecdsa_sha256(src,src_size))){
			return false;
		}
		sig->m_sign_size = 0;
		for(size_t i=0;i<sizeof(sig->m_sign_data);i++){
			#if ECDSA_RAND_FILL_SIGN_ENABLE
				sig->m_sign_data[i] = (uint8_t)rand();
			#else
				sig->m_sign_data[i] = 0;	
			#endif
		}
		
		mbedtls_ecdsa_context ctx_sign;
    
		mbedtls_ecdsa_init(&ctx_sign);
		if(!sissdk_ecdsa_load_privatekey(priKey,&ctx_sign)){
			mbedtls_ecdsa_free(&ctx_sign);
			return false;
		}
		
		sis_log(LogDebug,"[-s]Signing src hash...\r\n");
		int ret = 0;
		ret = ret;	//to disable warning
		size_t sig_size = 0;
    if ((ret = mbedtls_ecdsa_write_signature(&ctx_sign, MBEDTLS_MD_SHA256,
        pHash, SHA256_HASH_SIZE,
        sig->m_sign_data, &sig_size,
        NULL, NULL)) != 0)
    {
				sis_log(LogError,"[failed]:mbedtls_ecdsa_genkey returned %d\r\n", ret);
				mbedtls_ecdsa_free(&ctx_sign);
        return false;
    }
		sig->m_sign_size = sig_size;
		
    sis_log(LogDebug,"[ok]: ok (signature length = %u)\r\n", sig->m_sign_size);
		util_dump_buf("[ok]: Signature = ", sig->m_sign_data, sig->m_sign_size);
		
		mbedtls_ecdsa_free(&ctx_sign);
		return true;
}



